/*
 * evaldefs.h - Constants and types used in evaluating compiled shaders.
 *
 * RenderDotC (R) is:
 *      Copyright (c) 1996-2001 by Dot C Software, Inc.
 *      All Rights Reserved.
 * RenderDotC (R) is a registered trademark of Dot C Software, Inc.
 */

#ifndef EVALDEFS_H
#define EVALDEFS_H
#if defined(__sgi) || (_MSC_VER >= 1000)
#pragma once
#endif

#include <stddef.h>		/* For size_t */

/*
 * Enumerators
 */
#undef SO_TYPE
typedef enum {
    /*
     * Variable types
     */
    SO_TYPE_UNKNOWN,
    SO_TYPE_SCALAR,
    SO_TYPE_POINT,
    SO_TYPE_COLOR,
    SO_TYPE_STRING,

    /*
     * New types not in the 3.1 spec
     */
    SO_TYPE_VECTOR,
    SO_TYPE_NORMAL,
    SO_TYPE_MATRIX,

    /*
     * Shader types
     */
    SO_TYPE_SURFACE,
    SO_TYPE_LIGHT,
    SO_TYPE_DISPLACEMENT,
    SO_TYPE_VOLUME,
    SO_TYPE_TRANSFORMATION,
    SO_TYPE_IMAGER,

    /*
     * Internal types
     */
    SO_TYPE_LIGHTS
} SO_TYPE;

typedef enum {
    SO_STOR_UNKNOWN,
    SO_STOR_CONSTANT,
    SO_STOR_VARIABLE,
    SO_STOR_TEMPORARY,
    SO_STOR_PARAMETER,
    SO_STOR_GSTATE
} SO_STORAGE;

typedef enum {
    SO_DETAIL_UNKNOWN,
    SO_DETAIL_VARYING,
    SO_DETAIL_UNIFORM
} SO_DETAIL;

/*
 * SL Structure definitions
 */
typedef float SO_SCALAR;
typedef struct {
    float xval;
    float yval;
    float zval;
} SO_POINT;

struct SLvar {
    char *name;
    SO_TYPE type;
    SO_STORAGE storage;
    SO_DETAIL detail;
    char *spacename;
    void *defaultval;                   /* Pointer to default value */
    int arraylen;                       /* If array, number of elements */
    int isoutput;			/* True if declared "output" */
    size_t offset;
};

#endif
