/*
 * ri.h - header file for RenderMan Interface
 *
 * RenderDotC (R) is:
 *      Copyright (c) 1996-2001 by Dot C Software, Inc.
 *      All Rights Reserved.
 * RenderDotC (R) is a registered trademark of Dot C Software, Inc.
 *
 * The RenderMan (R) Interface Procedures and RIB Protocol are:
 *      Copyright 1988, 1989, Pixar.  All rights reserved.
 * RenderMan (R) is a registered trademark of Pixar.
 */

#ifndef RI_H
#define RI_H
#if defined(__sgi) || (_MSC_VER >= 1000)
#pragma once
#endif

#include "ridefs.h"                     /* For RtInt */

#undef RI_IMPORT
#undef RI_EXPORT
/*
 * Under Borland and Visual C++, use __declspec
 */
#if defined(__BORLANDC__) || defined(_MSC_VER)
    #define RI_EXPORT __declspec(dllexport)
    #define RI_IMPORT __declspec(dllimport)
/*
 * Otherwise, don't use
 */
#else
    #define RI_EXPORT
    #define RI_IMPORT
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern RtToken RI_IMPORT
        RI_FRAMEBUFFER, RI_FILE,
        RI_RGB, RI_RGBA, RI_RGBZ, RI_RGBAZ, RI_A, RI_Z, RI_AZ,
        RI_PERSPECTIVE, RI_ORTHOGRAPHIC,
        RI_HIDDEN, RI_PAINT,
        RI_CONSTANT, RI_SMOOTH,
        RI_FLATNESS, RI_FOV;

extern RtToken RI_IMPORT
        RI_AMBIENTLIGHT, RI_POINTLIGHT, RI_DISTANTLIGHT, RI_SPOTLIGHT,
        RI_INTENSITY, RI_LIGHTCOLOR, RI_FROM, RI_TO,
        RI_CONEANGLE, RI_CONEDELTAANGLE, RI_BEAMDISTRIBUTION,
        RI_MATTE, RI_METAL, RI_SHINYMETAL, RI_PLASTIC, RI_PAINTEDPLASTIC,
        RI_DEFAULTSURFACE, RI_KA, RI_KD, RI_KS, RI_ROUGHNESS, RI_KR,
        RI_TEXTURENAME, RI_SPECULARCOLOR,
        RI_DEPTHCUE, RI_FOG, RI_BUMPY,
        RI_MINDISTANCE, RI_MAXDISTANCE, RI_BACKGROUND,
        RI_DISTANCE, RI_AMPLITUDE;

extern RtToken RI_IMPORT
        RI_RASTER, RI_NDC, RI_SCREEN, RI_CAMERA, RI_WORLD, RI_OBJECT,
        RI_INSIDE, RI_OUTSIDE, RI_LH, RI_RH,
        RI_P, RI_PZ, RI_PW, RI_N, RI_NP,
        RI_CS, RI_OS, RI_S, RI_T, RI_ST,
        RI_BILINEAR, RI_BICUBIC,
        RI_PRIMITIVE, RI_INTERSECTION, RI_UNION, RI_DIFFERENCE,
        RI_PERIODIC, RI_NONPERIODIC, RI_CLAMP, RI_BLACK,
        RI_IGNORE, RI_PRINT, RI_ABORT, RI_HANDLER,
        RI_COMMENT, RI_STRUCTURE, RI_VERBATIM;

/*
 * Implementation specific:
 * Don't count on these being supported by other compliant renderers.
 */
extern RtToken RI_IMPORT
        RI_EYE,
        RI_IT, RI_MERGE, RI_ORIGIN,
        RI_TIFF, RI_RESOLUTION, RI_RESOLUTIONUNIT, RI_COMPRESSION,
        RI_LIMITS, RI_GRIDSIZE, RI_BUCKETSIZE, RI_TEXTUREMEMORY, RI_OPENMAPS,
        RI_ZTHRESHOLD, RI_EXTREMEDISPLACEMENT, RI_EYESPLITS,
        RI_SHADOW, RI_BIAS0, RI_BIAS1, RI_BIAS, RI_DEPTHFILTER,
        RI_SEARCHPATH, RI_SHADER, RI_TEXTURE, RI_DISPLAY, RI_ARCHIVE,
        RI_RESOURCE, RI_PROCEDURAL, RI_ENABLEGAUSSIAN, RI_ENABLELERP,
        RI_DICE, RI_BINARY,
        RI_BOUND, RI_DISPLACEMENT,
        RI_DISPLACEMENTBOUND, RI_COORDINATESYSTEM, RI_SPHERE, RI_SURFACE,
        RI_IDENTIFIER, RI_NAME, RI_SHADINGGROUP,
        RI_CHANLIST, RI_TYPE, RI_PATTERN, RI_NEWER, RI_MINMAX, RI_SIGNED,
	RI_REFWHITE,
	RI_TRIMCURVE, RI_SENSE, RI_PRIMITIVES, RI_NUPATCH, RI_ALL,
        RI_STATISTICS, RI_PROGRESS, RI_ENDOFFRAME, RI_DEBUG,
        RI_LINEAR, RI_CUBIC,
        RI_WIDTH, RI_CONSTANTWIDTH,
        RI_CATMARK, RI_LOOP, RI_HOLE, RI_CORNER, RI_CREASE,
	RI_INTERPBOUNDARY, RI_KNOTSPACING,
        RI_VOXEL, RI_DEPTH, RI_DENSITY, RI_CAPACITY,
        RI_RENDER, RI_MAX_RAYLEVEL, RI_GEOMMEMORY,
        RI_SPECULARBRDF, RI_PRMANSPECULAR, RI_STANDARD, RI_BMRT, RI_PRMAN,
        RI_RDCCLASSIC, RI_TORRANCE,
        RI_NOISE, RI_ALGORITHM, RI_PERLIN, RI_LEWIS, RI_IMPULSES,
        RI_VISIBILITY, RI_CASTS_SHADOWS, RI_NONE, RI_OPAQUE, RI_SHADE,
        RI_LIGHT, RI_SHADOWS, RI_ON, RI_OFF, RI_NSAMPLES,
	RI_MOTIONFACTOR, RI_SUBDIVISIONSTEPS, RI_SUBDIVIDEONLY;

/*
 * PRMan implementation specific: for compatibility only.
 * These have no effect.
 */
extern RtToken RI_IMPORT
        RI_RESIZE, RI_MOTIONFACTOR, RI_TRIMDEVIATION, RI_FBTRI;

/*
 * Cubic bases
 */
extern RtBasis RI_IMPORT
        RiBezierBasis, RiBSplineBasis, RiCatmullRomBasis,
        RiHermiteBasis, RiPowerBasis;

/*
 * Error number
 */
extern RtInt RI_IMPORT
        RiLastError;

/*
 * Declarations of All the RenderMan Interface Subroutines
 */
extern RtFloat RI_IMPORT
        RiGaussianFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth),
        RiBoxFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth),
        RiTriangleFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth),
        RiCatmullRomFilter(RtFloat x, RtFloat y,
                RtFloat xwidth, RtFloat ywidth),
        RiSincFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth),
        RiBesselFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth),
        RiDiskFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth),
        RiBartlettFilter(RtFloat x, RtFloat y, RtFloat xwidth, RtFloat ywidth);

extern RtVoid RI_IMPORT
        RiErrorIgnore(RtInt code, RtInt severity, char *msg),
        RiErrorPrint(RtInt code, RtInt severity, char *msg),
        RiErrorAbort(RtInt code, RtInt severity, char *msg);

extern RtVoid RI_IMPORT
        RiProcDelayedReadArchive(RtPointer data, RtFloat detail),
        RiProcRunProgram(RtPointer data, RtFloat detail),
        RiProcDynamicLoad(RtPointer data, RtFloat detail);

extern RtContextHandle RI_IMPORT RiGetContext(void);
extern RtVoid RI_IMPORT RiContext(RtContextHandle context);

extern RtToken RI_IMPORT
        RiDeclare(char *name, char *declaration);

extern RtVoid RI_IMPORT
        RiBegin(RtToken name),
        RiEnd(void),
        RiFrameBegin(RtInt frame),
        RiFrameEnd(void),
        RiWorldBegin(void),
        RiWorldEnd(void);

extern RtVoid RI_IMPORT
        RiFormat(RtInt xres, RtInt yres, RtFloat aspect),
        RiFrameAspectRatio(RtFloat aspect),
        RiScreenWindow(RtFloat left, RtFloat right, RtFloat bot, RtFloat top),
        RiCropWindow(RtFloat xmin, RtFloat xmax, RtFloat ymin, RtFloat ymax),
        RiProjection(RtToken name, ...),
        RiProjectionV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiClipping(RtFloat hither, RtFloat yon),
        RiDepthOfField(RtFloat fstop, RtFloat focallength,
                RtFloat focaldistance),
        RiShutter(RtFloat min, RtFloat max);

extern RtVoid RI_IMPORT
        RiPixelVariance(RtFloat variation),
        RiPixelSamples(RtFloat xsamples, RtFloat ysamples),
        RiPixelFilter(RtFilterFunc filterfunc, RtFloat xwidth, RtFloat ywidth),
        RiExposure(RtFloat gain, RtFloat gamma),
        RiImager(RtToken name, ...),
        RiImagerV(RtToken name, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiQuantize(RtToken type, RtInt one, RtInt min, RtInt max, RtFloat ampl),
        RiDisplay(char *name, RtToken type, RtToken mode, ...),
        RiDisplayV(char *name, RtToken type, RtToken mode,
                RtInt n, RtToken tokens[], RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiHider(RtToken type, ...),
        RiHiderV(RtToken type, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiColorSamples(RtInt n, RtFloat nRGB[], RtFloat RGBn[]),
        RiRelativeDetail(RtFloat relativedetail),
        RiOption(RtToken name, ...),
        RiOptionV(RtToken name, RtInt n, RtToken tokens[], RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiAttributeBegin(void),
        RiAttributeEnd(void),
        RiColor(RtColor color),
        RiOpacity(RtColor color),
        RiTextureCoordinates(RtFloat s1, RtFloat t1, RtFloat s2, RtFloat t2,
                RtFloat s3, RtFloat t3, RtFloat s4, RtFloat t4);

extern RtLightHandle RI_IMPORT
        RiLightSource(RtToken name, ...),
        RiLightSourceV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiAreaLightSource(RtToken name, ...),
        RiAreaLightSourceV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiIlluminate(RtLightHandle light, RtBoolean ononff),
        RiLightFree(RtLightHandle handle),
        RiSurface(RtToken name, ...),
        RiSurfaceV(RtToken name, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiAtmosphere(RtToken name, ...),
        RiAtmosphereV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiInterior(RtToken name, ...),
        RiInteriorV(RtToken name, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiExterior(RtToken name, ...),
        RiExteriorV(RtToken name, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiShadingRate(RtFloat size),
        RiShadingInterpolation(RtToken type),
        RiMatte(RtBoolean onoff);

extern RtVoid RI_IMPORT
        RiBound(RtBound bound),
        RiDetail(RtBound bound),
        RiDetailRange(RtFloat minvis, RtFloat lowtran, RtFloat uptran,
                RtFloat maxvis),
        RiGeometricApproximation(RtToken type, RtFloat value),
        RiOrientation(RtToken orientation),
        RiReverseOrientation(void),
        RiSides(RtInt sides);

extern RtVoid RI_IMPORT
        RiIdentity(void),
        RiTransform(RtMatrix transform),
        RiConcatTransform(RtMatrix transform),
        RiPerspective(RtFloat fov),
        RiTranslate(RtFloat dx, RtFloat dy, RtFloat dz),
        RiRotate(RtFloat angle, RtFloat dx, RtFloat dy, RtFloat dz),
        RiScale(RtFloat sx, RtFloat sy, RtFloat sz),
        RiSkew(RtFloat angle, RtFloat dx1, RtFloat dy1, RtFloat dz1,
                RtFloat dx2, RtFloat dy2, RtFloat dz2),
        RiDeformation(RtToken name, ...),
        RiDeformationV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiDisplacement(RtToken name, ...),
        RiDisplacementV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiCoordinateSystem(RtToken space),
        RiCoordSysTransform(RtToken space);

extern RtPointHandle RI_IMPORT
        RiTransformPoints(RtToken fromspace, RtToken tospace, RtInt n,
                RtPoint points[]);

extern RtVoid RI_IMPORT
        RiTransformBegin(void),
        RiTransformEnd(void);

extern RtVoid RI_IMPORT
        RiAttribute(RtToken name, ...),
        RiAttributeV(RtToken name, RtInt n, RtToken tokens[],
                RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiPolygon(RtInt nverts, ...),
        RiPolygonV(RtInt nverts, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiGeneralPolygon(RtInt nloops, RtInt nverts[], ...),
        RiGeneralPolygonV(RtInt nloops, RtInt nverts[], RtInt n,
                RtToken tokens[], RtPointer parms[]),
        RiPointsPolygons(RtInt npolys, RtInt nverts[], RtInt verts[], ...),
        RiPointsPolygonsV(RtInt npolys, RtInt nverts[], RtInt verts[], RtInt n,
                RtToken tokens[], RtPointer parms[]),
        RiPointsGeneralPolygons(RtInt npolys, RtInt nloops[], RtInt nverts[],
                RtInt verts[], ...),
        RiPointsGeneralPolygonsV(RtInt npolys, RtInt nloops[], RtInt nverts[],
                RtInt verts[], RtInt n, RtToken tokens[], RtPointer parms[]),
        RiBasis(RtBasis ubasis, RtInt ustep, RtBasis vbasis, RtInt vstep),
        RiPatch(RtToken type, ...),
        RiPatchV(RtToken type, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiPatchMesh(RtToken type, RtInt nu, RtToken uwrap, RtInt nv,
                RtToken vwrap, ...),
        RiPatchMeshV(RtToken type, RtInt nu, RtToken uwrap, RtInt nv,
                RtToken vwrap, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiNuPatch(RtInt nu, RtInt uorder, RtFloat uknot[], RtFloat umin,
                RtFloat umax, RtInt nv, RtInt vorder, RtFloat vknot[],
                RtFloat vmin, RtFloat vmax, ...),
        RiNuPatchV(RtInt nu, RtInt uorder, RtFloat uknot[], RtFloat umin,
                RtFloat umax, RtInt nv, RtInt vorder, RtFloat vknot[],
                RtFloat vmin, RtFloat vmax, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiTrimCurve(RtInt nloops, RtInt ncurves[], RtInt order[],
                RtFloat knot[], RtFloat min[], RtFloat max[], RtInt n[],
                RtFloat u[], RtFloat v[], RtFloat w[]);

extern RtVoid RI_IMPORT
        RiPoints(RtInt nverts, ...),
        RiPointsV(RtInt nverts, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiCurves(RtToken type, RtInt ncurves, RtInt nvertices[], RtToken wrap,
                ...),
        RiCurvesV(RtToken type, RtInt ncurves, RtInt nvertices[], RtToken wrap,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiNuCurves(RtInt ncurves, RtInt nvertices[], RtInt order[],
                RtFloat knot[], RtFloat min[], RtFloat max[], ...),
        RiNuCurvesV(RtInt ncurves, RtInt nvertices[], RtInt order[],
                RtFloat knot[], RtFloat min[], RtFloat max[], RtInt n,
                RtToken tokens[], RtPointer parms[]),
        RiSubdivisionMesh(RtToken scheme, RtInt nfaces, RtInt nvertices[],
            RtInt vertices[], RtInt ntags, RtToken tags[], RtInt nargs[],
            RtInt intargs[], RtFloat floatargs[], ...),
        RiSubdivisionMeshV(RtToken scheme, RtInt nfaces, RtInt nvertices[],
            RtInt vertices[], RtInt ntags, RtToken tags[], RtInt nargs[],
            RtInt intargs[], RtFloat floatargs[], RtInt n, RtToken tokens[],
            RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiSphere(RtFloat radius, RtFloat zmin, RtFloat zmax, RtFloat tmax, ...),
        RiSphereV(RtFloat radius, RtFloat zmin, RtFloat zmax, RtFloat tmax,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiCone(RtFloat height, RtFloat radius, RtFloat tmax, ...),
        RiConeV(RtFloat height, RtFloat radius, RtFloat tmax, RtInt n,
                RtToken tokens[], RtPointer parms[]),
        RiCylinder(RtFloat radius, RtFloat zmin, RtFloat zmax, RtFloat tmax,
                ...),
        RiCylinderV(RtFloat radius, RtFloat zmin, RtFloat zmax, RtFloat tmax,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiHyperboloid(RtPoint point1, RtPoint point2, RtFloat tmax, ...),
        RiHyperboloidV(RtPoint point1, RtPoint point2, RtFloat tmax, RtInt n,
                RtToken tokens[], RtPointer parms[]),
        RiParaboloid(RtFloat rmax, RtFloat zmin, RtFloat zmax, RtFloat tmax,
                ...),
        RiParaboloidV(RtFloat rmax, RtFloat zmin, RtFloat zmax, RtFloat tmax,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiDisk(RtFloat height, RtFloat radius, RtFloat tmax, ...),
        RiDiskV(RtFloat height, RtFloat radius, RtFloat tmax, RtInt n,
                RtToken tokens[], RtPointer parms[]),
        RiTorus(RtFloat majrad, RtFloat minrad, RtFloat phimin, RtFloat phimax,
                RtFloat tmax, ...),
        RiTorusV(RtFloat majrad, RtFloat minrad, RtFloat phimin, RtFloat phimax,
                RtFloat tmax, RtInt n, RtToken tokens[], RtPointer parms[]),
        RiBlobby(RtInt nleaf, RtInt ninst, RtInt inst[], RtInt nflt,
            RtFloat flt[], RtInt nstr, RtToken str[], ...),
        RiBlobbyV(RtInt nleaf, RtInt ninst, RtInt inst[], RtInt nflt,
            RtFloat flt[], RtInt nstr, RtToken str[], RtInt n,
            RtToken tokens[], RtPointer parms[]),
        RiProcedural(RtPointer data, RtBound bound, RtSubdivideFunc subdivfunc,
                RtFreeFunc freefunc),
        RiGeometry(RtToken type, ...),
        RiGeometryV(RtToken type, RtInt n, RtToken tokens[], RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiSolidBegin(RtToken operation),
        RiSolidEnd(void);

extern RtObjectHandle RI_IMPORT
        RiObjectBegin(void);

extern RtVoid RI_IMPORT
        RiObjectEnd(void),
        RiObjectInstance(RtObjectHandle handle),
        RiObjectFree(RtObjectHandle handle),
        RiMotionBegin(RtInt n, ...),
        RiMotionBeginV(RtInt n, RtFloat times[]),
        RiMotionEnd(void);

extern RtVoid RI_IMPORT
        RiResource(RtToken handle, RtToken type, ...),
        RiResourceV(RtToken handle, RtToken type, RtInt n, RtToken tokens[],
                RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiMakeTexture(char *pic, char *tex, RtToken swrap, RtToken twrap,
                RtFilterFunc filterfunc, RtFloat swidth, RtFloat twidth, ...),
        RiMakeTextureV(char *pic, char *tex, RtToken swrap, RtToken twrap,
                RtFilterFunc filterfunc, RtFloat swidth, RtFloat twidth,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiMakeBump(char *pic, char *tex, RtToken swrap, RtToken twrap,
                RtFilterFunc filterfunc, RtFloat swidth, RtFloat twidth, ...),
        RiMakeBumpV(char *pic, char *tex, RtToken swrap, RtToken twrap,
                RtFilterFunc filterfunc, RtFloat swidth, RtFloat twidth,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiMakeLatLongEnvironment(char *pic, char *tex, RtFilterFunc filterfunc,
                RtFloat swidth, RtFloat twidth, ...),
        RiMakeLatLongEnvironmentV(char *pic, char *tex, RtFilterFunc filterfunc,
                RtFloat swidth, RtFloat twidth, RtInt n, RtToken tokens[],
                RtPointer parms[]),
        RiMakeCubeFaceEnvironment(char *px, char *nx, char *py, char *ny,
                char *pz, char *nz, char *tex, RtFloat fov,
                RtFilterFunc filterfunc, RtFloat swidth, RtFloat twidth, ...),
        RiMakeCubeFaceEnvironmentV(char *px, char *nx, char *py, char *ny,
                char *pz, char *nz, char *tex, RtFloat fov,
                RtFilterFunc filterfunc, RtFloat swidth, RtFloat twidth,
                RtInt n, RtToken tokens[], RtPointer parms[]),
        RiMakeShadow(char *pic, char *tex, ...),
        RiMakeShadowV(char *pic, char *tex, RtInt n, RtToken tokens[],
                RtPointer parms[]);

extern RtVoid RI_IMPORT
        RiErrorHandler(RtErrorHandler handler);

extern RtVoid RI_IMPORT
        RiArchiveRecord(RtToken type, char *fmt, ...),
        RiReadArchive(RtToken name, RtCallbackFunc callback, ...),
        RiReadArchiveV(RtToken name, RtCallbackFunc callback, RtInt n,
                RtToken tokens[], RtPointer parms[]);

#ifdef __cplusplus
}
#endif

#endif
