/*
 * ridefs.h - Definitions and types used in RenderMan Interface.
 *
 * RenderDotC (R) is:
 *      Copyright (c) 1996-2001 by Dot C Software, Inc.
 *      All Rights Reserved.
 * RenderDotC (R) is a registered trademark of Dot C Software, Inc.
 *
 * The RenderMan (R) Interface Procedures and RIB Protocol are:
 *      Copyright 1988, 1989, Pixar.  All rights reserved.
 * RenderMan (R) is a registered trademark of Pixar.
 */

#ifndef RIDEFS_H
#define RIDEFS_H
#if defined(__sgi) || (_MSC_VER >= 1000)
#pragma once
#endif

/*
 * Abstract types used in RenderMan Interface.
 */
typedef short            RtBoolean;
#if defined(__alpha) && defined(__osf__) || defined(_MIPS_SZLONG) && (_MIPS_SZLONG > 32)
typedef int              RtInt;
#else
typedef long             RtInt;
#endif
typedef float            RtFloat;

typedef char            *RtToken;

typedef RtFloat          RtColor[3];
typedef RtFloat          RtPoint[3];
typedef RtFloat          RtPointw[4];
typedef RtFloat          RtMatrix[4][4];
typedef RtFloat          RtBasis[4][4];
typedef RtFloat          RtBound[6];
typedef char            *RtString;

typedef void            *RtPointer;
#define RtVoid           void

typedef RtVoid (*RtSubdivideFunc)(RtVoid *, RtFloat);
typedef RtVoid (*RtFreeFunc)(RtVoid *);
typedef RtVoid (*RtCallbackFunc)(RtToken, char *, ...);
typedef RtVoid (*RtErrorHandler)(RtInt, RtInt, char *);
typedef RtFloat (*RtFilterFunc)(RtFloat, RtFloat, RtFloat, RtFloat);

typedef RtPointer        RtObjectHandle;
typedef RtPointer        RtLightHandle;
typedef RtPoint         *RtPointHandle;
typedef RtPointer        RtContextHandle;

/*
 * Extern Declarations for Predefined RI Data Structures
 */
#define RI_FALSE        0
#define RI_TRUE         (!RI_FALSE)
#define RI_INFINITY     1.0e38
#define RI_EPSILON      1.0e-10
#define RI_NULL         ((RtToken)0)

#define RI_BEZIERSTEP           ((RtInt)3)
#define RI_BSPLINESTEP          ((RtInt)1)
#define RI_CATMULLROMSTEP       ((RtInt)1)
#define RI_HERMITESTEP          ((RtInt)2)
#define RI_POWERSTEP            ((RtInt)4)

/*
 *      Error Codes
 *
 *       1 - 10         System and File Errors
 *      11 - 20         Program Limitations
 *      21 - 40         State Errors
 *      41 - 60         Paramter and Protocol Errors
 *      61 - 80         Execution Errors
 */
#define RIE_NOERROR         ((RtInt)0)

#define RIE_NOMEM           ((RtInt)1)  /* Out of memory */
#define RIE_SYSTEM          ((RtInt)2)  /* Miscellaneous system error */
#define RIE_NOFILE          ((RtInt)3)  /* File nonexistent */
#define RIE_BADFILE         ((RtInt)4)  /* Bad file format */
#define RIE_VERSION         ((RtInt)5)  /* File version mismatch */
#define RIE_DISKFULL        ((RtInt)6)  /* Target disk is full */

#define RIE_INCAPABLE       ((RtInt)11) /* Optional RI feature */
#define RIE_UNIMPLEMENT     ((RtInt)12) /* Unimplemented feature */
#define RIE_LIMIT           ((RtInt)13) /* Arbitrary program limit */
#define RIE_BUG             ((RtInt)14) /* Probably a bug in renderer */

#define RIE_NOTSTARTED      ((RtInt)23) /* RiBegin not called */
#define RIE_NESTING         ((RtInt)24) /* Bad begin-end nesting */
#define RIE_NOTOPTIONS      ((RtInt)25) /* Invalid state for options */
#define RIE_NOTATTRIBS      ((RtInt)26) /* Invalid state for attribs */
#define RIE_NOTPRIMS        ((RtInt)27) /* Invalid state for primitives */
#define RIE_ILLSTATE        ((RtInt)28) /* Other invalid state */
#define RIE_BADMOTION       ((RtInt)29) /* Badly formed motion block */
#define RIE_BADSOLID        ((RtInt)30) /* Badly formed solid block */

#define RIE_BADTOKEN        ((RtInt)41) /* Invalid token for request */
#define RIE_RANGE           ((RtInt)42) /* Parameter out of range */
#define RIE_CONSISTENCY     ((RtInt)43) /* Parameters inconsistent */
#define RIE_BADHANDLE       ((RtInt)44) /* Bad object/light handle */
#define RIE_NOSHADER        ((RtInt)45) /* Can't load requested shader */
#define RIE_MISSINGDATA     ((RtInt)46) /* Required parameters not provided */
#define RIE_SYNTAX          ((RtInt)47) /* Declare type syntax error */

#define RIE_MATH            ((RtInt)61) /* Zerodivide, noninvert matrix, etc. */

/*
 * Error severity levels
 */
#define RIE_INFO            ((RtInt)0)  /* Rendering stats and other info */
#define RIE_WARNING         ((RtInt)1)  /* Something seems wrong, maybe okay */
#define RIE_ERROR           ((RtInt)2)  /* Problem.  Results may be wrong */
#define RIE_SEVERE          ((RtInt)3)  /* So bad you should probably abort */

#endif
