/*
 * so.h - header file for interrogating compiled shaders in RenderDotC
 *
 * RenderDotC (R) is:
 *      Copyright (c) 1996-2001 by Dot C Software, Inc.
 *      All Rights Reserved.
 * RenderDotC (R) is a registered trademark of Dot C Software, Inc.
 *
 * The RenderMan (R) Interface Procedures and RIB Protocol are:
 *      Copyright 1988, 1989, Pixar.  All rights reserved.
 * RenderMan (R) is a registered trademark of Pixar.
 */

#ifndef SO_H
#define SO_H
#if defined(__sgi) || (_MSC_VER >= 1000)
#pragma once
#endif

#include "evaldefs.h"

#undef RI_IMPORT
#undef RI_EXPORT
/*
 * Under Borland and Visual C++, use __declspec
 */
#if defined(__BORLANDC__) || defined(_MSC_VER)
    #define RI_EXPORT __declspec(dllexport)
    #define RI_IMPORT __declspec(dllimport)
/*
 * Otherwise, don't use
 */
#else
    #define RI_EXPORT
    #define RI_IMPORT
#endif

typedef struct sovissymdef {
    char *svd_name;
    SO_TYPE svd_type;
    SO_STORAGE svd_storage;             /* Storage class */
    SO_DETAIL svd_detail;               /* Uniform or varying */
    char *svd_spacename;                /* Space for points and colors */
    union {                             /* Pointer to default value */
        SO_POINT *pointval;
        SO_SCALAR *scalarval;
        char *stringval;
        float *matrixval;
    } svd_default;
    union svd_defaultvalu {             /* Inline default value */
        SO_POINT svd_pointval;
        SO_SCALAR svd_scalarval;
    } svd_defaultval;
    int svd_valisvalid;                 /* True if default value is available */
    int svd_arraylen;                   /* If array, number of elements */
} SO_VISSYMDEF;

#define NULL_SOVISSYMDEF 0

/*
 * Exported functions
 */
#ifdef __cplusplus
extern "C" {
#endif

extern void RI_IMPORT So_SetPath(const char *path);
extern char *RI_IMPORT So_GetPath(void);
extern int RI_IMPORT So_SetShader(const char *name);
extern char *RI_IMPORT So_GetName(void);
extern SO_TYPE RI_IMPORT So_GetType(void);
extern int RI_IMPORT So_GetNArgs(void);
extern SO_VISSYMDEF *RI_IMPORT So_GetArgByName(const char *name);
extern SO_VISSYMDEF *RI_IMPORT So_GetArgById(int id);
extern SO_VISSYMDEF *RI_IMPORT So_GetArrayArgElement(const SO_VISSYMDEF *array,
    int index);
extern void RI_IMPORT So_EndShader(void);
extern char *RI_IMPORT So_TypetoStr(SO_TYPE type);
extern char *RI_IMPORT So_StortoStr(SO_STORAGE storage);
extern char *RI_IMPORT So_DetailtoStr(SO_DETAIL detail);

#ifdef __cplusplus
}
#endif

#endif
